import tkinter as tk
from tkinter import ttk
from tkinter import messagebox
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from tkinter import filedialog
from matplotlib import colormaps

# start the main app:
root = tk.Tk()
root.title('BAGI alpha')
root.geometry('760x500')
root.iconbitmap('Icon.ico')

# define the color palette. This can be based on https://materialui.co/colors
colors = {
    'foreground': '#ffffff',
    'background': '#EEEEEE',
    'accent': '#FFCC80',
    'text': 'black'}

# configure the background:
root.configure(bg=colors['background'], padx=0, pady=0)

# create and configure a style to be used in tabs and labels
s = ttk.Style()
s.theme_create("BAGI_theme", parent="alt", settings={
    'TLabel': {
        'configure': {
            'background': colors['foreground'],
            'foreground': colors['text'],
            'padding': [0, 5, 7, 0]}},
    'TFrame': {
        'configure': {
            'background': colors['foreground'],
            'borderwidth': 0,
            'highlightcolor': 'red'}},
    'TMenubutton': {
        'configure': {
            'background': colors['accent'],
            'foreground': colors['text'],
            'padding': [5, 0, 0, 0],
            'width': '30'
        }},
    'TMenu': {
        'configure': {
            'background': colors['accent'],
            'foreground': colors['text'],
            'width': '20',
            'arrowcolor': colors['text'],
        }},

})

s.configure('BAGI_theme', relief='flat',
            background=colors['background'],
            foreground=colors['text'],
            insertcolor=colors['text'],
            selectbackground=colors['accent']
            )

s.theme_use("BAGI_theme")


def save_image():
    ftypes = [('PNG file', '.png'), ('SVG file', '.svg'), ('All files', '*')]
    filename = filedialog.asksaveasfilename(filetypes=ftypes, defaultextension='.png')
    # save the plot in the specified path; the 'tight' option removes the whitespace from around the figure:
    plt.savefig(filename, bbox_inches='tight', dpi=300)


def reset_scores():
    for criterion in criteria:
        criterion.valuevar.set(2.5)
    for dropdown in [dropdown1, dropdown2, dropdown3, dropdown4, dropdown5, dropdown6, dropdown7, dropdown8,
                     dropdown9, dropdown10]:
        dropdown.dropdown_var.set('Select')
    create_plot()

def call_info_popup():
    win = tk.Toplevel()
    win_frame = tk.Frame(win, width=500, background=colors['foreground'])
    win_frame.pack()
    win.iconbitmap('Icon.ico')
    win.wm_title('About and citation info')
    text = ttk.Label(win_frame, text='If you use BAGI, please cite:', wraplength=280, justify='left')
    text.grid(column=0, row=0, padx=8, pady=8, sticky='w')
    text2 = tk.Text(win_frame, width=50, height=5, wrap='word', bg=colors['background'])
    citation = 'N. Manousi, W. Wojnowski, J. Płotka-Wasylka, V. Samanidou, Blue Applicability Grade Index'\
                   '(BAGI) and Software: A new tool for the evaluation of method’s practicality, ' \
                   'doi.org/XX.XXXX/XXXXXXXXXX'
    text2.grid(column=0, row=1, padx=8, pady=8, sticky='w')
    text2.insert(tk.END, citation)
    text3 = ttk.Label(win_frame, text='Most recent version of the source code can be found at:', wraplength=320, justify='left')
    text3.grid(column=0, row=2, padx=8, pady=8, sticky='w')
    text4 = tk.Text(win_frame, width=50, height=1, wrap='word', bg=colors['background'])
    source_link = 'git.pg.edu.pl/p174235/bagi'
    text4.grid(column=0, row=3, padx=8, pady=8, sticky='w')
    text4.insert(tk.END, source_link)
    text5 = ttk.Label(win_frame, text='© 2023 N. Manousi, W. Wojnowski, J. Płotka-Wasylka, V. Samanidou 2023, '
                                      'available under the MIT License', wraplength=380,
                      justify='left')
    text5.grid(column=0, row=4, padx=8, pady=8, sticky='w')



def create_menu():
    menu = tk.Menu()
    root.config(menu=menu)
    file_menu = tk.Menu(menu)
    file_menu.config(background=colors['background'],
                     activeborderwidth=5,
                     activeforeground=colors['text'],
                     foreground=colors['text'],
                     tearoff=0)

    file_menu.add_command(label='Save image', command=save_image)
    file_menu.add_command(label='Re-set', command=reset_scores)
    about_menu = tk.Menu(menu)
    about_menu.config(background=colors['background'],
                      activeborderwidth=5,
                      activeforeground=colors['text'],
                      foreground=colors['text'],
                      tearoff=0)
    menu.add_cascade(label='File', menu=file_menu)
    menu.add_cascade(label='About', menu=about_menu)
    about_menu.add_command(label='Info', command=call_info_popup)


create_menu()

'''
Coordinates of the polygon vertices
'''
coordinates = {
    0: [(0, -14.833), (14.107, -4.584), (8.719, 12), (-8.719, 12), (-14.107, -4.584)],
    1: [(8.719, 12), (21.796, 30.00), (-21.796, 30), (-8.719, 12)],
    2: [(-8.719, 12), (-21.794, 30), (-35.267, -11.459), (-14.107, -4.584)],
    3: [(-14.107, -4.584), (-35.267, -11.459), (0, -37.082), (0, -14.833)],
    4: [(0, -14.833), (0, -37.082), (35.267, -11.459), (14.107, -4.584)],
    5: [(35.267, -11.459), (14.107, -4.584), (8.719, 12.00), (21.796, 30)],

    6: [(20.497, 34), (0, 62.22), (-20.497, 34)],
    7: [(-26, 30), (-59.179, 19.215), (-38.67, -8.987)],
    8: [(-36.578, -15.474), (-36.578, -50.345), (-3.4, -39.554)],
    9: [(36.578, -15.474), (36.578, -50.345), (3.4, -39.554)],
    10: [(26, 30), (59.179, 19.215), (38.67, -8.987)],
}

colors_list = {
    2.5: 'white',
    5.0: (0.6768166089965398, 0.8164705882352941, 0.9023760092272203, 1.0),
    7.5: (0.22380622837370245, 0.5375317185697808, 0.7584313725490196, 1.0),
    10.0: (0.03137254901960784, 0.18823529411764706, 0.4196078431372549, 1.0),
}

# create the two main frames of the GUI
left_frame = tk.Frame(root, bd=0, padx=2, pady=0, width=500, height=500, bg=colors['foreground'],
                      highlightbackground=colors['background'], highlightthickness=8)
left_frame.pack(side='left', anchor='n', fill='both', expand=True)

right_frame = tk.Frame(root, bd=0, padx=2, pady=0, width=500, height=500, bg=colors['background'],
                       highlightbackground=colors['background'], highlightthickness=8,
                       highlightcolor=colors['background'])
right_frame.pack(side='right', anchor='n')


class Criterion:

    def __init__(self, number):
        self.number = number
        self.valuevar = tk.DoubleVar()
        self.valuevar.set(2.5)
        self.optionvar = tk.StringVar()
        self.optionvar.set('select')
        self.coordinates = coordinates[number]


'''
Create the 10 criteria values (11 counting the central field). They can be
later called as list elements, e.g. to get the value of criterion 3:
    criteria[2].textvar.get()
'''

criteria = []
for i in range(0, 11):
    criteria.append(Criterion(i))

'''
a function for restarting the right frame (to clear the memory after each change)
it is then used when there is a need to change the plot
'''


def clear_frame(frame):
    frame.destroy()
    global right_frame
    right_frame = tk.Frame(root, bd=0, padx=2, pady=0, width=500, height=500, bg=colors['foreground'],
                           highlightbackground=colors['background'], highlightthickness=8,
                           highlightcolor=colors['background'])
    right_frame.pack(side='right', anchor='n')


def calculate_score():
    score = 0
    for criterion in criteria[1:]:
        score += criterion.valuevar.get()

    return round(score, 1)


def create_plot(event=None):
    # the event=None is passed so that the entry.bind
    # does not return a positional argument;
    # close the current plot window (otherwise the generated plots remain open in the memory):
    plt.close()

    # re-generate the frame:
    clear_frame(right_frame)

    fig, ax = plt.subplots(figsize=(5, 5), dpi=150, facecolor=colors['foreground'])
    for criterion in criteria[1:]:
        polygon = plt.Polygon(criterion.coordinates,
                              facecolor=colors_list[criterion.valuevar.get()],
                              edgecolor='black')

        ax.add_patch(polygon)

    ax.autoscale()
    ax.set_aspect('equal')
    center_score = calculate_score()

    if center_score == 25:
        central_polygon = plt.Polygon(criteria[0].coordinates,
                                      facecolor='white',
                                      edgecolor='black')
    else:
        central_polygon = plt.Polygon(criteria[0].coordinates,
                                      facecolor=colormaps['Blues'](int(255 * (center_score - 25) / 75)),
                                      edgecolor='black')
    ax.add_patch(central_polygon)

    if center_score >= 80:
        ax.text(0, 0, str(center_score), ha='center', va='center', fontsize=10, color='white')
    else:
        ax.text(0, 0, str(center_score), ha='center', va='center', fontsize=10, color='black')

    ax.axis('off')

    canvas = FigureCanvasTkAgg(figure=fig, master=right_frame)
    plot_widget = canvas.get_tk_widget()
    plot_widget.pack(side='top', padx=8, pady=8)


class Dropdown:

    def __init__(self, criterion, options, label_text, row):
        self.var = criterion.valuevar
        self.options = options
        self.row = row
        self.dropdown_var = tk.StringVar()
        self.dropdown_var.set('Select')
        self.dropdown = ttk.OptionMenu(left_frame, self.dropdown_var, self.dropdown_var.get(), *self.options.keys())
        self.dropdown.grid(row=self.row + 1, column=0, padx=8, sticky='W')
        self.dropdown_var.trace('w', self.change_dropdown)
        self.label = ttk.Label(left_frame, text=label_text)
        self.label.grid(row=self.row, column=0, padx=8, sticky='W')

    def change_dropdown(self, *args):
        self.var.set(self.options[self.dropdown_var.get()])
        create_plot()

'''
Populate the dropdowns
'''

dropdown1 = Dropdown(criterion=criteria[1], options={
    'Qualitative': 2.5,
    'Screening': 5.0,
    'Quantitative': 7.5,
    'Quantitative and Confirmatory': 10.0}, row=1,
                     label_text='1. Type of Analysis')

dropdown2 = Dropdown(criterion=criteria[2], options={
    'Single Element': 2.5,
    'Multi-element analysis for 2-5 compounds of the same chemical class': 5.0,
    'Multi-element analysis for 6-15 compounds of the same chemical group or 2-15 compounds of different chemical '
    'classes': 7.5,
    'Multi-element analysis for > 15 compounds': 10.0}, row=3,
                     label_text='2. Multi- or single-element analysis')

dropdown3 = Dropdown(criterion=criteria[3], options={
    'Instrumentation that is not commonly available in most labs (SFC, 2D-GC, 2D-LC, LC-MS/MS, GC-MS/MS, etc.)': 2.5,
    'Sophisticated instrumentation (LC-MS, GC-MS, ICP-MS, homemade interfaces, homemade automatic systems, etc.)': 5.0,
    'Simple instrumentation available in most labs (UV, HPLC-UV, HPLC-DAD, UHPLC, FAAS, ETAAS, ICP-OES, GC-FID etc.)': 7.5,
    'Simple in operation  portable instrumentation (smart-phone based detectors, portable GC, etc.)': 10.0},
    row=5, label_text='3. Analytical technique')

dropdown4 = Dropdown(criterion=criteria[4], options={
    '1': 2.5,
    '2-12': 5.0,
    '13-95': 7.5,
    '≥96': 10.0},
    row=7, label_text='4. Simultaneous sample preparation')

dropdown5 = Dropdown(criterion=criteria[5], options={
    'Multi-step sample preparation required (e.g. LLE, SPE and/or derivatization)': 2.5,
    'Miniaturized extraction sample preparation (SPME, DLLME, MEPS, SBSE, d-SPE, FPSE, etc.)': 5.0,
    'Simple, low-cost sample preparation required (e.g. protein precipitation)': 7.5,
    'Not required or on-site sample preparation if required': 10.0},
    row=9, label_text='5. Sample preparation')

dropdown6 = Dropdown(criterion=criteria[6], options={
    '≤1': 2.5,
    '2-4': 5.0,
    '5-10': 7.5,
    '>10': 10.0},
    row=11, label_text='6. Samples per hour')

dropdown7 = Dropdown(criterion=criteria[7], options={
    'Need to be synthesized in the lab with advanced equipment or know-how (specially designed metal-organic frameworks, modified nanomaterials, etc.)': 2.5,
    'Need to be synthesized in the lab with common instrumentation and in a simple way': 5.0,
    'Commercially available reagents not common in QC labs (derivatization reagents, SPE cartridges, SPME fibers, etc.)': 7.5,
    'Common commercially available reagents (e.g. methanol, acetonitrile, HNO3, etc.)': 10.0},
    row=13, label_text='7. Reagents and materials')

dropdown8 = Dropdown(criterion=criteria[8], options={
    'Preconcentration required. Legislation criteria met after complicated stages (e.g. extraction, evaporation, and reconstitution).': 2.5,
    'Preconcentration required. Required sensitivity is met with one-step preconcentration.': 7.5,
    'No preconcentration required. Required sensitivity and /or legislation criteria are met directly.': 10.0},
    row=15, label_text='8. Fit-for-purpose')

dropdown9 = Dropdown(criterion=criteria[9], options={
    'Manual treatment and analysis': 2.5,
    'Semi-automated with non-common devices (e.g. homemade systems)': 5.0,
    'Semi-automated with common devices (e.g. HPLC autosampler)': 7.5,
    'Fully automated with novel technology advanced devices (robotics, lab-in-syringe, etc.)': 10.0},
    row=17, label_text='9. Degree of automation')

dropdown10 = Dropdown(criterion=criteria[10], options={
    '>1000 μL (or mg) bioanalytical samples; >100 mL (or g) food/environmental': 2.5,
    '501-1000 μL (or mg) bioanalytical samples; 51-100 mL (or g) food/environmental': 5.0,
    '100-500 μL (or mg) bioanalytical samples; 10-50 ml (or g) food/environmental': 7.5,
    '<100 μL (or mg) bioanalytical samples; <10 mL (or g) food/environmental': 10.0},
    row=19, label_text='10. Amount of sample')

# start the GUI with a default plot
create_plot()


def main():
    while True:
        root.mainloop()


main()

